@@ -1,4 +1,4 @@
-// Copyright (C) 1999-2003 Open Source Telecom Corporation.
+// Copyright (C) 1999-2005 Open Source Telecom Corporation.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -14,29 +14,26 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 // 
-// As a special exception to the GNU General Public License, permission is 
-// granted for additional uses of the text contained in its release 
-// of Common C++.
-// 
-// The exception is that, if you link the Common C++ library with other
-// files to produce an executable, this does not by itself cause the
-// resulting executable to be covered by the GNU General Public License.
-// Your use of that executable is in no way restricted on account of
-// linking the Common C++ library code into it.
+// As a special exception, you may use this file as part of a free software
+// library without restriction.  Specifically, if other files instantiate
+// templates or use macros or inline functions from this file, or you compile
+// this file and link it with other files to produce an executable, this
+// file does not by itself cause the resulting executable to be covered by
+// the GNU General Public License.  This exception does not however    
+// invalidate any other reasons why the executable file might be covered by
+// the GNU General Public License.    
 //
-// This exception does not however invalidate any other reasons why
-// the executable file might be covered by the GNU General Public License.
-// 
-// This exception applies only to the code released under the 
-// name Common C++.  If you copy code from other releases into a copy of
+// This exception applies only to the code released under the name GNU
+// Common C++.  If you copy code from other releases into a copy of GNU
 // Common C++, as the General Public License permits, the exception does
 // not apply to the code that you add in this way.  To avoid misleading
 // anyone as to the status of such modified files, you must delete
 // this exception notice from them.
-// 
-// If you write modifications of your own for Common C++, it is your choice
+//
+// If you write modifications of your own for GNU Common C++, it is your choice
 // whether to permit this exception to apply to your modifications.
 // If you do not wish that, delete this exception notice.
+//
 
 /**
  * @file misc.h
@@ -72,6 +69,9 @@
 namespace ost {
 #endif
 
+class __EXPORT Runlist;
+class __EXPORT Runable;
+
 /**
  * The memory pager is used to allocate cumulative memory pages for
  * storing object specific "persistant" data that is presumed to persist
@@ -407,10 +407,8 @@ public:
 	 * default system values with user specific keyword values.
 	 *
 	 * @param keypath (filepath/section)
-	 * @param environment string
 	 */ 
-	void load(const char *keypath, 
-		  const char *environment = "CONFIG_KEYDATA");
+	void load(const char *keypath);
 
 	/**
 	 * Load additional key values into the currrent object from
@@ -424,11 +422,8 @@ public:
 	 *
 	 * @param prefix
 	 * @param keypath (filepath/section)
-	 * @param environment optional environment variable that
-	 * specifies the home path.
 	 */ 
-	void loadPrefix(const char *prefix,
-		  const char *keypath, const char *environment = "CONFIG_KEYDATA");
+	void loadPrefix(const char *prefix, const char *keypath);
 
 	/**
 	 * Load additional keys into the current object using a real
@@ -461,10 +456,9 @@ public:
 	 * initial config file section into it.
 	 *
 	 * @param keypath (filepath/section)
-	 * @param environment optional environment variable that
 	 * specifies the home path.
 	 */
-	Keydata(const char *keypath, const char *environment="CONFIG_KEYDATA");
+	Keydata(const char *keypath);
 
 	/**
 	 * Destroy the keydata object and all allocated memory.  This
@@ -614,6 +608,132 @@ public:
 	inline void operator delete[](void *) {};
 };
 
+/**
+ * This class is used to associate (object) pointers with named strings.
+ * A virtual is used to allocate memory which can be overriden in the
+ * derived class.
+ *
+ * @author David Sugar <dyfet@ostel.com>
+ * @short associate names with pointers.
+ */
+class __EXPORT Assoc
+{
+private:
+	class __EXPORT entry
+	{
+	public:
+		const char *id;
+		entry *next;
+		void *data;
+	};
+
+	entry *entries[KEYDATA_INDEX_SIZE];
+
+protected:
+	Assoc();
+
+	void clear(void);
+
+	virtual void *getMemory(size_t size) = 0;
+
+public:
+	void *getPointer(const char *id);
+	void setPointer(const char *id, void *data);
+};
+
+/**
+ * A runlist is used to restrict concurrent exection to a limited set
+ * of concurrent sessions, much like a semaphore.  However, the runlist
+ * differs in that it notifies objects when they become ready to run,
+ * rather than requiring them to wait and "block" for the semaphore
+ * count to become low enough to continue.
+ *
+ * @author David Sugar <dyfet@ostel.com>
+ * @short list of runable objects.
+ */
+class __EXPORT Runlist : public Mutex
+{
+private:
+	Runable *first, *last;
+
+protected:
+	unsigned limit, used;
+	void check(void);
+
+public:
+	/**
+	 * Create a new runlist with a specified limit.
+	 *
+	 * @param limit before wait queuing.
+	 */
+	Runlist(unsigned count = 1);
+	
+	/**
+	 * Add a runable object to this runlist.  If the number of
+	 * entries running is below the limit, then add returns true
+	 * otherwise the entry is added to the list.
+	 *
+	 * @return true if immediately ready to run
+	 * @param runable object.
+	 */
+	bool add(Runable *run);
+
+	/**
+	 * Remove a runable object from the wait list or notify when
+	 * it is done running so that the used count can be decremented.
+	 *
+	 * @param runable object.
+	 */
+	void del(Runable *run);
+
+	/**
+	 * Set the limit.
+	 *
+	 * @param limit to use.
+	 */
+	void set(unsigned limit);
+};
+
+/**
+ * A container for objects that can be queued against a runlist.
+ *
+ * @author David Sugar <dyfet@ostel.com>
+ * @short runable object with notify when ready.
+ */
+class __EXPORT Runable
+{
+private:
+	friend class __EXPORT Runlist;
+	Runlist *list;
+	Runable *next, *prev;
+
+protected:
+	Runable();
+	virtual ~Runable();
+
+	/**
+	 * Method handler that is invoked when a wait-listed object
+	 * becomes ready to run.
+	 */
+	virtual void ready(void) = 0;
+
+public:
+	/** 
+	 * Start the object against a run list.
+	 *
+	 * @return true if immediately available to run.
+	 * @param list to start under.
+	 */
+	bool starting(Runlist *list);
+	
+	/**
+	 * Stop the object, called when stopping or ready completes.
+	 * May also be used for a task that has not yet started to
+	 * remove it from the wait list.
+	 */
+	void stoping(void);
+};
+
 #ifdef	CCXX_NAMESPACES
 }
 #endif
