@@ -1111,6 +1111,8 @@ inline void ThreadImpl::ThreadExecHandler(Thread *th)
 	th->setCancel(Thread::cancelDisabled);
 
 	pthread_cleanup_pop(0);
+	if(th->isDetached())
+		ThreadImpl::_self.setKey(NULL);
 	th->close();
 	pthread_exit(NULL);
 }
